/******************************************************************************
 *
 * Copyright (c) 1999-2003 AppGate Network Security AB. All Rights Reserved.
 * 
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the MindTerm Public Source License,
 * Version 2.0, (the 'License'). You may not use this file except in compliance
 * with the License.
 * 
 * You should have received a copy of the MindTerm Public Source License
 * along with this software; see the file LICENSE.  If not, write to
 * AppGate Network Security AB, Otterhallegatan 2, SE-41118 Goteborg, SWEDEN
 *
 *****************************************************************************/

package com.mindbright.examples;

import java.net.Socket;

import com.mindbright.jca.security.SecureRandom;
import com.mindbright.util.SecureRandomAndPad;
import com.mindbright.util.RandomSeed;

import com.mindbright.ssh2.SSH2Transport;
import com.mindbright.ssh2.SSH2SimpleClient;
import com.mindbright.ssh2.SSH2ConsoleRemote;

/**
 * This is a simple demo of running a single command on a ssh2 server in exactly
 * the same way an ordinary unix ssh2 client works when passed a command-line to
 * be run.
 * <p>
 * <pre>
 * usage: RunRemoteCommand &lt;server[:port]&gt; &lt;username&gt; &lt;password&gt; &lt;command-line&gt;
 * </pre>
 *
 * @see RunRemoteCommand2
 */
public class RunRemoteCommand {

    public static void main(String[] argv) {
	if(argv.length < 4) {
	    System.out.println("usage: RunRemoteCommand <server[:port]> <username> <password> <command-line>");
	    System.exit(1);
	}
	try {
	    String server = argv[0];
	    String user   = argv[1];
	    String passwd = argv[2];
	    int    port   = 22;

	    int i = server.indexOf(':');
	    if(i != -1) {
		String p = server.substring(i + 1);
		server = server.substring(0, i);
		port = Integer.parseInt(p);
	    }

	    String cmdLine = "";
	    for(i = 3; i < argv.length; i++) {
		cmdLine += argv[i] + " ";
	    }
	    cmdLine = cmdLine.trim();

	    /*
	     * Connect to the server and authenticate using plain password
	     * authentication (if other authentication method needed check other
	     * constructors for SSH2SimpleClient).
	     */
	    Socket serverSocket     = new Socket(server, port);
	    SSH2Transport transport = new SSH2Transport(serverSocket,
							createSecureRandom());
	    SSH2SimpleClient client = new SSH2SimpleClient(transport,
							   user, passwd);

	    /*
	     * Create the remote console to use for command execution.
	     */
	    SSH2ConsoleRemote console =
		new SSH2ConsoleRemote(client.getConnection());

	    /*
	     * Run the command (returns a boolean indicating success, we ignore
	     * it here). Here we redirect stdout and stderr of the remote command
	     * execution to our own stdout and stderr for simplicity.
	     */
	    console.command(cmdLine, System.out, System.err);

	    /*
	     * Retrieve the exit status of the command (from the remote end).
	     */
	    int exitStatus = console.waitForExitStatus();

	    /*
	     * NOTE: at this point System.out will be closed together with the
	     * session channel of the console
	     */

	    /*
	     * Disconnect the transport layer gracefully
	     */
	    transport.normalDisconnect("User disconnects");

	    /*
	     * Exit with same status as remote command did
	     */
	    System.exit(exitStatus);

	} catch (Exception e) {
	    System.out.println("An error occured: " + e);
	    System.exit(1);
	}
    }

    public static SecureRandomAndPad createSecureRandom() {
	/* NOTE, this is how it should be done if you want good
	 * randomness, however good randomness takes time so we settle with
	 * just some low-entropy garbage here.
	 *
	 * RandomSeed seed = new RandomSeed("/dev/random", "/dev/urandom");
	 * byte[] s = seed.getBytesBlocking(20);
	 * return new SecureRandomAndPad(new SecureRandom(s));
	 *
	 */
	byte[] seed = RandomSeed.getSystemStateHash();
	return new SecureRandomAndPad(new SecureRandom(seed));
    }

}
