/******************************************************************************
 *
 * Copyright (c) 1999-2003 AppGate Network Security AB. All Rights Reserved.
 * 
 * This file contains Original Code and/or Modifications of Original Code as
 * defined in and that are subject to the MindTerm Public Source License,
 * Version 2.0, (the 'License'). You may not use this file except in compliance
 * with the License.
 * 
 * You should have received a copy of the MindTerm Public Source License
 * along with this software; see the file LICENSE.  If not, write to
 * AppGate Network Security AB, Otterhallegatan 2, SE-41118 Goteborg, SWEDEN
 *
 *****************************************************************************/

package com.mindbright.examples;

import java.net.Socket;
import java.io.IOException;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import com.mindbright.terminal.TerminalWin;
import com.mindbright.terminal.TerminalXTerm;
import com.mindbright.terminal.LineReaderTerminal;
import com.mindbright.terminal.GlobalClipboard;
import com.mindbright.terminal.TerminalFrameTitle;
import com.mindbright.net.telnet.TelnetTerminalAdapter;

/**
 * Demonstrates the terminal window. This class opens a pure telnet
 * connection to anywhere. SSH is not involved at all.
 */
public class Telnet extends WindowAdapter {

    private Socket                socket;
    private String                remoteHost;
    private int                   remotePort;
    private LineReaderTerminal    lineReader;
    private TelnetTerminalAdapter telnetAdapter;

    public Telnet() {
	this(null, -1);
    }

    public Telnet(String remoteHost) {
	setFromHostString(remoteHost);
    }

    public Telnet(String remoteHost, int remotePort) {
	this.remoteHost = remoteHost;
	this.remotePort = remotePort;
    }

    public void startMeUp() {
	Frame       frame    = new Frame();
	TerminalWin terminal = new TerminalWin(frame, new TerminalXTerm());

	terminal.setClipboard(GlobalClipboard.getClipboardHandler());

	frame.setLayout(new BorderLayout());
	frame.add(terminal.getPanelWithScrollbar(), BorderLayout.CENTER);

	TerminalFrameTitle frameTitle =
	    new TerminalFrameTitle(frame, "Telnet (not connected)");
	frameTitle.attach(terminal);

	frame.addWindowListener(this);

	frame.pack();
	frame.show();

	try {
	    if(remoteHost == null) {
		lineReader = new LineReaderTerminal(terminal);
		String host = null;
		do {
		    host = lineReader.promptLine("\r\nremote host[:port] : ",
						 null, false);
		} while(host.trim().length() == 0);
		setFromHostString(host);
		lineReader.detach();
	    }

	    socket = new Socket(remoteHost, remotePort);

	    telnetAdapter = new TelnetTerminalAdapter(socket.getInputStream(),
						      socket.getOutputStream(),
						      terminal);

	    frameTitle.setTitleName("telnet@" + remoteHost + (remotePort != 23 ?
							      (":" + remotePort)
							      : ""));

	    telnetAdapter.getTelnetNVT().getThread().join();

	    if(telnetAdapter.isBuffered()) {
		lineReader = new LineReaderTerminal(terminal);
		lineReader.promptLine("\n\rTelnet session was closed, press <return> to close window", null,
				      false);
	    }

	} catch (Exception e) {
	    System.out.println("Error: " + e.getMessage());
	} finally {
	    frame.dispose();
	}
    }

    private void setFromHostString(String remoteHost) {
	if(remoteHost == null) {
	    return;
	}
	int i = remoteHost.indexOf(':');
	if(i != -1) {
	    try {
		this.remotePort = Integer.parseInt(remoteHost.substring(i + 1));
	    } catch (Exception e) {
		this.remotePort = 23;
	    }
	    remoteHost = remoteHost.substring(0, i);
	} else {
	    this.remotePort = 23;
	}
	this.remoteHost = remoteHost;
    }

    public static void main(String[] argv) {
	if(argv.length > 1) {
	    System.out.println("usage: Telnet <server[:port]>");
	    System.exit(1);
	}
	Telnet telnet = new Telnet(argv.length > 0 ? argv[0] : null);
	telnet.startMeUp();
	System.exit(0);
    }

    public void windowClosing(WindowEvent e) {
	if(socket != null) {
	    try { socket.close(); } catch (IOException ee) { /* don't care */ }
	}
	if(lineReader != null) {
	    lineReader.breakPromptLine("");
	}
    }

}
