/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Created on Jul 1, 2005 at 10:05:09 AM.
 */
package uk.ac.standrews.cs.nds.eventModel;

/**
 * Defines utility methods for creating particular kinds of events.
 *
 * @author graham
 */
public class EventFactory {

	/**
	 * Type of a diagnostic event.
	 */
	public static final String DIAGNOSTIC_EVENT = "DiagnosticEvent";

	/**
	 * Attribute  name for diagnostic event message.
	 */
	public static final String DIAGNOSTIC_EVENT_MSG = "msg";

	/**
	 * Type of an error event.
	 */
	public static final String ERROR_EVENT = "ErrorEvent";

	/**
	 * Attribute name for error event message.
	 */
	public static final String ERROR_EVENT_MSG = "msg";

	/**
	 * Creates a diagnostic event.
	 *
	 * @param message the diagnostic message
	 * @return the event
	 */
	public static Event makeDiagnosticEvent(String message) {

		Event event = new Event(DIAGNOSTIC_EVENT);
		event.put(DIAGNOSTIC_EVENT_MSG, message);
		return event;
	}

	/**
	 * Creates an error event.
	 *
	 * @param message the error message
	 * @return the event
	 */
	public static Event makeErrorEvent(String message) {

		Event event = new Event(ERROR_EVENT);
		event.put(ERROR_EVENT_MSG, message);
		return event;
	}
}
