/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 01-Nov-2004
 */
package uk.ac.standrews.cs.nds.eventModel.eventBus;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.standrews.cs.nds.eventModel.IEvent;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventBus;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventConsumer;

/**
 * @author stuart, graham
 */
public class EventBus implements IEventBus {

	private final Collection<IEventConsumer> registered_consumers;

	/**
	 * Creates a new event bus.
	 */
	public EventBus() {

		registered_consumers = new ArrayList<IEventConsumer>();
	}

	/**
	 * Distributes the given event to all interested consumers.
	 *
	 * @param event the event to be distributed
	 */
	public void publishEvent(IEvent event) {

		for (IEventConsumer event_consumer : registered_consumers)
			if (event_consumer.interested(event)) event_consumer.receiveEvent(event);
	}

	/**
	 * Registers a new consumer with the event bus.
	 *
	 * @param consumer the event consumer to be registered
	 */
	public void register(IEventConsumer consumer) {

		registered_consumers.add(consumer);
	}

	/**
	 * Removes a consumer from the event bus.
	 *
	 * @param consumer the event consumer to be removed
	 */
	public void unregister(IEventConsumer consumer) {

		registered_consumers.remove(consumer);
	}
}
