/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 01-Nov-2004
 */
package uk.ac.standrews.cs.nds.eventModel.eventBus.test;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import java.util.HashSet;
import java.util.Set;

import org.junit.Test;

import uk.ac.standrews.cs.nds.eventModel.Event;
import uk.ac.standrews.cs.nds.eventModel.IEvent;
import uk.ac.standrews.cs.nds.eventModel.eventBus.EventBus;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventBus;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventConsumer;

/**
 * @author stuart
 */
public class EventBusTest {

	/**
	 *
	 */
	@Test
	public void eventBusDelivery() {

		IEventBus bus = new EventBus();

		Consumer consumer1 = new Consumer("type1");
		Consumer consumer2 = new Consumer("type2");
		Consumer consumer3 = new Consumer("type3");

		IEvent event1 = new Event("type1");
		IEvent event2 = new Event("type2");
		IEvent event3 = new Event("type3");

		bus.register(consumer1);
		bus.register(consumer2);
		bus.register(consumer3);

		assertFalse(consumer1.getEventsReceived().contains(event1));

		bus.publishEvent(event1);

		assertTrue(consumer1.getEventsReceived().contains(event1));
		assertFalse(consumer2.getEventsReceived().contains(event1));

		bus.publishEvent(event2);

		assertTrue(consumer2.getEventsReceived().contains(event2));
		assertFalse(consumer1.getEventsReceived().contains(event2));

		bus.unregister(consumer3);

		bus.publishEvent(event3);

		assertFalse(consumer3.getEventsReceived().contains(event3));
	}
}

class Consumer implements IEventConsumer {

	private final Set<IEvent> events_received;
	private final String type_interested_in;

	Consumer(String type_interested_in) {

		this.type_interested_in = type_interested_in;
		events_received = new HashSet<IEvent>();
	}

	public boolean interested(IEvent event) {

		return event.getType().equals(type_interested_in);
	}

	public void receiveEvent(IEvent event) {

		events_received.add(event);
	}

	Set<IEvent> getEventsReceived() {

		return events_received;
	}
}