/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 23-Jun-2005
 */
package uk.ac.standrews.cs.nds.p2p.impl;

import uk.ac.standrews.cs.nds.p2p.exceptions.P2PNodeException;
import uk.ac.standrews.cs.nds.p2p.interfaces.INextHopResult;

/**
 * @author stuart
 */
public abstract class AbstractNextHopResult implements INextHopResult{

	// TODO introduce type parameterisation to all this stuff

	private NextHopResultStatus code;
	private P2PNodeException error;
	private Object appObject;

	/**
	 * @param code
	 */
	public AbstractNextHopResult(NextHopResultStatus code) {
		this(code, null);
	}

	/**
	 * @param code
	 * @param appObject
	 */
	public AbstractNextHopResult(NextHopResultStatus code, Object appObject) {
		this.code = code;
		this.appObject = appObject;
		error = null;
	}

	/**
	 * @param error
	 */
	public AbstractNextHopResult(P2PNodeException error) {
		code = NextHopResultStatus.ERROR;
		this.error = error;
		appObject= null;
	}

	/**
	 * @return the code
	 */
	public NextHopResultStatus getCode() {
		return code;
	}

	/**
	 * @return the error
	 */
	public P2PNodeException getError() {
		return error;
	}

	/**
	 * @param errorMsg Describes the error that occurred.
	 */
	public void setError(String errorMsg){
		code = NextHopResultStatus.ERROR;
		error = new P2PNodeException(P2PStatus.APPLICATION_FAILURE,errorMsg);
	}

	public Object getAppObject(){
		return appObject;
	}

	/**
	 * @param appObject the object to set
	 * @param root true if the result is the root
	 */
	public void setAppObject(Object appObject, boolean root) {

		code = root ? NextHopResultStatus.ROOT : NextHopResultStatus.INTERCEPT;

		this.appObject = appObject;
	}

	/**
	 * @return true if an error has occurred.
	 */
	public boolean isError() {
		return code == NextHopResultStatus.ERROR;
	}
}
