/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 20-Mar-2006
 */
package uk.ac.standrews.cs.nds.p2p.impl;

import uk.ac.standrews.cs.nds.p2p.exceptions.P2PNodeException;
import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PRouter;

/**
 * stu - This class currently provides a level of indirection for when
 * a P2PNode's IP2PRoutingAPI interface is made remotely accessible. This
 * is handy during debugging because it allows for the explicit intercept
 * of remote calls. In the longer term I think we might implement extra
 * access controls here.
 */
public class P2PRoutingGateway implements IP2PRouter {

	private final IP2PRouter node;

	/**
	 * @param node
	 */
	public P2PRoutingGateway(IP2PRouter node){
		this.node=node;
	}

	public IP2PNode lookup(IKey k) throws P2PNodeException {
		return node.lookup(k);
	}

	public DOLResult dol(IKey k, AID appID) throws P2PNodeException {
		return node.dol(k,appID);
	}

	public DOLResult dol_noIntercept(IKey k, AID appID) throws P2PNodeException {
		return node.dol_noIntercept(k,appID);
	}
}
