/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 06-Jun-2005
 */
package uk.ac.standrews.cs.nds.p2p.impl;

/**
 * @author stuart
 */
public class P2PStatus {

	private final String statusMsg;

	/**
	 * Instantiation failure.
	 */
	public static final P2PStatus INSTANTIATION_FAILURE = new P2PStatus("Failed to instantiate P2P node.");

	/**
	 * Hostname initialisation failure.
	 */
	public static final P2PStatus LOCAL_HOST_INIT_FAILURE = new P2PStatus("Could not configure P2P node with specified hostname or IP address.");

	/**
	 * Port initialisation failure.
	 */
	public static final P2PStatus LOCAL_PORT_INIT_FAILURE = new P2PStatus("Could not configure P2P node with specified port.");

	/**
	 * Known node access failure.
	 */
	public static final P2PStatus KNOWN_NODE_FAILURE = new P2PStatus("No P2P node could be found at specified known node address.");

	/**
	 * Join failure.
	 */
	public static final P2PStatus NODE_JOIN_FAILURE = new P2PStatus("P2P node could not join P2P network using specified known node address.");

	/**
	 * Node instantiated.
	 */
	public static final P2PStatus NODE_RUNNING = new P2PStatus("P2P node is running.");

	/**
	 * Node failure.
	 */
	public static final P2PStatus NODE_FAILED = new P2PStatus("P2P node has failed.");

	/**
	 * Node deployment failure.
	 */
	public static final P2PStatus GENERAL_NODE_DEPLOYMENT_FAILURE = new P2PStatus("Failed to instantiate and deploy P2P node.");

	/**
	 * Service deployment failure.
	 */
	public static final P2PStatus SERVICE_DEPLOYMENT_FAILURE = new P2PStatus("Failed to deploy P2P node's network services.");

	//lookup failures
	public static final P2PStatus LOOKUP_FAILURE = new P2PStatus("A failure occured during execution of P2P key lookup");

	//local node failures
	public static final P2PStatus STATE_ACCESS_FAILURE = new P2PStatus("A failure occured while acessing the local P2P node's local state.");

	//application level failure
	public static final P2PStatus APPLICATION_FAILURE = new P2PStatus("A failure occured in an application level object.");

	private P2PStatus(String statusMsg){
		this.statusMsg=statusMsg;
	}

	public String getStatusMsg() {
		return statusMsg;
	}
}
