/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on Jan 18, 2005 at 3:42:43 PM.
 */
package uk.ac.standrews.cs.nds.p2p.interfaces;

import uk.ac.standrews.cs.nds.p2p.exceptions.P2PNodeException;

/**
 * Generic interface provided by a node in a P2P network.
 *
 * @author al, graham, stuart
 */
public interface IP2PNode extends IP2PNodeRepresentation, Comparable<IP2PNode> {

	/**
	 * Finds the live (root) node to which the given key currently maps. This corresponds to (part of) the Key Based Routing abstraction.
	 *
	 * @param k a key
	 * @return the P2P node to which the key maps
	 * @throws P2PNodeException if the node could not be located
	 */
	public IP2PNode lookup(IKey k) throws P2PNodeException;

	/**
	 * Returns an indication of the size of the current routing state. Currently used in simulations; will need to be generalised
	 * as part of overall monitoring infrastructure for autonomic control.
	 *
	 * @return an indication of the size of the current routing state
	 */
	public int routingStateSize();

	/**
	 * Tests whether the node is simulating failure.
	 *
	 * @return true if the node is simulating failure
	 */
	public boolean isSimulatingFailure();
}
