/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 10-May-2005
 */
package uk.ac.standrews.cs.nds.p2p.interfaces;

import uk.ac.standrews.cs.nds.p2p.exceptions.P2PNodeException;
import uk.ac.standrews.cs.nds.p2p.impl.AID;
import uk.ac.standrews.cs.nds.p2p.impl.DOLResult;

/**
 * Defines a set of P2P routing methods available to application-level components.
 *
 * @author stuart, graham
 */
public interface IP2PRouter {

    /**
     * Finds the live (root) P2P node to which the given key currently maps. This corresponds to (part of) the Key Based Routing abstraction.
     *
     * @param k a key
     * @return the P2P node to which the key maps
     *
     * @throws P2PNodeException if the node could not be located
     */
    public IP2PNode lookup(IKey k) throws P2PNodeException;

    // ********************* Decentralised object location methods. *********************

    /**
     * Finds an application-level component, with a given Application ID, that has responsibility
     * for a given key. This is the first application-level component that indicates interest in
     * the key, along the route to the root node of the key.
     *
     * @param k a key
     * @param appID an Application ID
     * @return a handle to the first matching application-level component that indicates interest in the key
     *
     * @throws P2PNodeException if a suitable application-level component could not be located.
     */
    public DOLResult dol(IKey k, AID appID) throws P2PNodeException;

    /**
     * Returns the application-level component, with a given Application ID, at the root
     * node of a given key. This operation is similar to {@link #dol(IKey, AID)} but without the
     * opportunity for 'early return' if a component on an intermediate node is interested in the key.
     *
     * @param k a key
     * @param appID an Application ID
     * @return a handle to the matching application-level component on the root node for the key
     *
     * @throws P2PNodeException if the application-level component could not be located
     */
    public DOLResult dol_noIntercept(IKey k, AID appID) throws P2PNodeException;
}
