/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on Dec 15, 2004 at 11:36:19 AM.
 */
package uk.ac.standrews.cs.nds.p2p.simulation.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.util.ReverseIterator;

/**
 * @author al
 *
 *  Route provides a data structure to hold routing information from some node for some key.
 */
public class Route<T extends IP2PNode> implements Iterable<T> {

    private List<T> hops;
    private IKey searchKey;
    private T start;

    public Route( IKey searchKey ) {
        this(null, searchKey);
    }

    /**
     *  Construct a Route object
     */
    public Route( T start, IKey searchKey ) {
        hops = new ArrayList<T>();
        this.start = start;
        this.searchKey = searchKey;
    }

    /**
     * @param next the next node to add to route table
     */
    public void addHop(T next) {
        hops.add(next);
    }

    public void addHopAtStart(T next) {
        hops.add(0,next);
    }

    public T lastHop() {
        return hops.get( hops.size() - 1 );
    }

    /**
     * @return the number of hops in the route
     */
    public int hop_count() {
        return hops.size();
    }

    /**
     * @return an iterator over the hops in the route
     */
    public Iterator<T> iterator() {
        return hops.iterator();
    }

    /**
     * @return a reverse iterator over the hops in the route
     */
    public Iterator<T> reverseIterator() {
        return new ReverseIterator<T>(hops);
    }

    /**
     * @return the search key.
     */
    public IKey getSearchKey() {
        return searchKey;
    }

    /**
     * @return the start node
     */
    public T getStart() {
        return start;
    }

    public Route<T> commonPath( Route<T> other ) {
        if( this.searchKey.compareTo( other.searchKey ) == 0 ) {
            Route<T> r = new Route<T>(this.searchKey);
            int index = hops.size() - 1; // last entry in the array list.
            int other_index = other.hops.size() - 1; // last entry in other array list
            while( index >= 0 &&  other_index >= 0 )
				// run down array list comparing
                if( hops.get(index).equals( other.hops.get(other_index))) {
                    r.addHopAtStart(hops.get(index));
                    index--;
                    other_index--;
                } else
					break;
        	return r;
        }
        return null; // in the event of failure
    }
}
