/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.p2p.simulation.util;

import java.util.Enumeration;
import java.util.HashMap;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;

import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.p2p.simulation.impl.Route;
import uk.ac.standrews.cs.nds.p2p.simulation.interfaces.IP2PSimulation;
import uk.ac.standrews.cs.nds.p2p.util.SHA1KeyFactory;
import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;
import uk.ac.standrews.cs.nds.util.ErrorHandling;

/**
 * @author mjl
 */
public class BottomUpSpanTreeGen<T extends IP2PNode> {
    private final HashMap<IKey, TreeNode> sofar;

	private final IP2PSimulation<T> simulation;
    /**
     * @param simulation
     */
    public BottomUpSpanTreeGen(IP2PSimulation<T> simulation) {
    	//super(simulation);
    	this.simulation=simulation;
    	sofar = new HashMap<IKey, TreeNode>();
    }

	public DefaultMutableTreeNode computeAllPathsToRandomNode() {
	    Diagnostic.trace(DiagnosticLevel.RUN);
	    IKey k = new SHA1KeyFactory().generateKey(Integer.toString(simulation.getRand().nextInt()));

	//    IP2PNode node = simulation.getNodes().get(1);
		Route<T> rt = simulation.makeRoute(simulation.getNodes().get(1), k );    // route from 1 to k.
		//Route<?> rt = simulation.makeRoute(node, k );    // route from 1 to k.
	    IKey rootKey = null;
        try {
            rootKey = rt.lastHop().getKey();
        } catch (Exception e) {
            ErrorHandling.exceptionError(e, "error getting node representation");
        }

	    DefaultMutableTreeNode  tree = new DefaultMutableTreeNode( rt.lastHop() );  // set up a new tree - root is the target of route
	    for( int i = 0 ; i < simulation.getNodes().size() ; i++ ) //if(i!=rootIndex)
		{
				rt = simulation.makeRoute( simulation.getNodes().get(i),k ); // find a route to k from node i
				addRouteToTree( rootKey ,tree,rt );
				simulation.showRoute( simulation.getNodes().get(i),k,rt );
			}
	    return tree;
	}

	protected void addRouteToTree( IKey target, DefaultMutableTreeNode tree, Route<T> rt ) {
		try {
			if( tree == null || rt == null )
				ErrorHandling.hardError("Empty root/route passed");
			if( !sofar.containsKey(target) ) sofar.put( target, tree.getRoot() );
			rt.addHopAtStart(rt.getStart());
			DefaultMutableTreeNode tail = null;

			for (IP2PNode j : rt) {

				IKey key = j.getKey();
				if( sofar.containsKey(key) ) {
					if( tail != null ) ((DefaultMutableTreeNode) sofar.get(key)).add(tail);
					return;
				}
				DefaultMutableTreeNode t = new DefaultMutableTreeNode(j);
				if( tail != null ) t.add(tail);
				tail = t;
				sofar.put( key, tail );
			}
			ErrorHandling.hardError("Roots do not match");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    protected DefaultMutableTreeNode addNodeToTree(IP2PNode j, DefaultMutableTreeNode tree){
    	  IKey nextKey = null;
        try {
            nextKey = j.getKey();
        } catch (Exception e) {
            ErrorHandling.exceptionError(e, "error getting node representation");
        }
        // lookup key in current node and see if Chord node is in tree already
          DefaultMutableTreeNode child = findChild( tree,nextKey );
          if( child == null ) { // next node is not in tree so add it as a child of current tree
              child = new DefaultMutableTreeNode( j );
              tree.add( child );
          }
          return child;
    }

    /**
     * @param tree a DefaultMutableTreeNode containing a partially build tree
     * @param key - the key for which are searching.
     * @return a child of node tree containing a ChordNode with key key
     */
    protected DefaultMutableTreeNode findChild(DefaultMutableTreeNode tree, IKey key) {
        Enumeration<?> children = tree.children();
        while( children.hasMoreElements() ) {
            DefaultMutableTreeNode next = (DefaultMutableTreeNode) children.nextElement();
            IP2PNode nextNode = (IP2PNode) next.getUserObject();
            try {
                if( key.compareTo( nextNode.getKey() ) == 0 )
					return next;
            } catch (Exception e) { ErrorHandling.exceptionError(e, "error getting node representation"); }
        }
        // we haven't found it.
        return null;
    }
}