/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.p2p.simulation.util;

import java.util.Enumeration;

import javax.swing.tree.DefaultMutableTreeNode;

import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.p2p.simulation.interfaces.IP2PSimulation;
import uk.ac.standrews.cs.nds.util.ErrorHandling;

/**
 * @author al
 */
public class GraphGen{

	private final IP2PSimulation<?> simulation;
    /**
     * @param simulation
     */
    public GraphGen(IP2PSimulation<?> simulation) {
        this.simulation=simulation;
    }

    /**
     * generates a dot file for GraphViz tool
     * @param tree - the tree to visualise.
     */
    public void show_dot_file(DefaultMutableTreeNode tree, String label) {
        //get the index for the root node
        IP2PNode jcn = (IP2PNode)tree.getUserObject();
        int root = 0;
        try {
            root = simulation.key2Index(jcn.getKey());
        } catch (Exception e) {
            ErrorHandling.exceptionError(e, "ChordNode.getKey threw exception");
        }
        System.out.println( "digraph \"\" {" ); // the first thing in a dot file
        System.out.println( "node[style=filled,width=.125,height=.375,fontsize=9];");
        System.out.println( "graph[overlap=false,fontname=\"Helvetica-Oblique\",fontsize=12,label=\""+label+"\"];");
        show_paths( tree );
        show_ranking(root,5);
        show_colouringRGB(tree,tree.getDepth(),0x811480,0xFE0906,0xFDE800);
        System.out.println( "}" ); // the last thing in a dot file
    }

    private void show_colouringRGB(DefaultMutableTreeNode tree, int depth, int colourFar, int colourNear, int rootColour) {
        int colourBase = colourFar > colourNear ? colourNear : colourFar;
        int colourRange = Math.abs(colourFar-colourNear);
        //int colourRange = colourFar > colourNear ? colourFar-colourNear : colourNear-colourFar;
        int colourInc=colourRange/depth;

        Enumeration<?> e = tree.depthFirstEnumeration();
        while(e.hasMoreElements()){
            DefaultMutableTreeNode current = (DefaultMutableTreeNode) e.nextElement();
            IP2PNode jcn=(IP2PNode) current.getUserObject();
            int level=current.getLevel();
            int index = 0;
            try {
                index = simulation.key2Index(jcn.getKey());
            } catch (Exception e1) {
                ErrorHandling.exceptionError(e1, "error getting node representation");
            }
            int colourVal;
            if(e.hasMoreElements()){
                if(colourInc!=0)
					colourVal=colourBase+level*colourInc;
				else
					colourVal=colourBase;
            } else
				colourVal=rootColour;
            String str = Integer.toHexString( colourVal  & 0xffffff);
            String colourStr = "#" + "000000".substring( str.length() ) + str.toUpperCase();

            System.out.println(index + " [color=\"" + colourStr  +"\"];");
        }
    }

	/**
     *
     */
    private void show_ranking(int root, int distanceInc) {
    	int length = simulation.getNodes().size();
    	int count=length/2;
    	System.out.print("{rank=same; ");
    	for(int i=0;i<=count;i++){

    		if(i%distanceInc==1)
				System.out.print("{rank=same; ");

    		int leftIndex=root-i;
			if(leftIndex<0)
				leftIndex+=length;

			int rightIndex=root+i;
			if(rightIndex>=length)
				rightIndex-=length;

			System.out.print(leftIndex + " ");
			if(rightIndex!=leftIndex)
				System.out.print(rightIndex + " ");

			if(i%distanceInc==0)
				System.out.print(";}\n");
		}
    }

    /**
     * generates nodes for dot file for GraphViz tool
     * @param tree - the subtree to visualise.
     */
    private void show_paths(DefaultMutableTreeNode tree) {
        try {
            if (tree != null) {
            	IP2PNode nextNode = (IP2PNode) tree.getUserObject();
                int rootrep = simulation.key2Index(nextNode.getKey());
                Enumeration<?> children = tree.children();
                while (children.hasMoreElements()) {
                    DefaultMutableTreeNode next = (DefaultMutableTreeNode) children.nextElement();
                    IP2PNode childNode = (IP2PNode) next.getUserObject();
                    int childrep = simulation.key2Index(childNode.getKey());
                    System.out.println(childrep + " -> " + rootrep);
                    show_paths(next);
                }
            }
        } catch (Exception e) {
            ErrorHandling.exceptionError(e, "error getting node representation");
        }
    }
}
