/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.p2p.simulation.util;

import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.p2p.simulation.impl.Route;
import uk.ac.standrews.cs.nds.p2p.simulation.interfaces.IP2PSimulation;
import uk.ac.standrews.cs.nds.p2p.util.SHA1KeyFactory;
import uk.ac.standrews.cs.nds.util.ErrorHandling;

/**
 * @author stuart
 */
public class PathDistance<T extends IP2PNode> {

    public double total_hops;
    public double total_distance;
	private IP2PSimulation<T> simulation;

    /**
     * @param simulation
     */
    public PathDistance(IP2PSimulation<T> simulation) {
        this.simulation=simulation;
    }

    /**
     * @return the length of the route rt in real distance.
     */
    public double pathlength(Route<?> rt) {

        double result = 0.0;
        IP2PNode first = rt.getStart();

        for (IP2PNode node : rt) {

            try {
                result += simulation.getDistanceCalculator().distance(first.getAddress().getAddress(), node.getAddress().getAddress());
            }
            catch (Exception e) { ErrorHandling.exceptionError(e, "error getting node representation"); }
        }
        return result;
    }

    public void analyseAllPathsToRandomNode() {
    	total_hops = 0;
        total_distance = 0;
        IKey k = new SHA1KeyFactory().generateKey(Integer.toString(simulation.getRand().nextInt()));
        for( int i = 0 ; i < simulation.getNodeCount() ; i++ ) {
            Route<?> rt = simulation.makeRoute( simulation.getNodes().get(i),k ); // find a route to k from node i

            int rt_hops=0;
            double rt_len = 0.0;
            //ignore the route from the target node back to itself [A->B->C->D->A]
            if(rt.hop_count()>0 && rt.lastHop()!=rt.getStart()){
            	rt_hops = rt.hop_count();
            	rt_len = pathlength( rt );
            }
            total_hops += rt_hops;
            total_distance += rt_len;
        }
    }
}
