/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.p2p.simulation.util;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.util.ErrorHandling;

public class TreeViewGUI extends JFrame {

	private String rootTag;
	/* SET GUI FONT */
	private final int FONT_SIZE = 9;
	private final Font FONT = new java.awt.Font("Verdana",
			Font.PLAIN, FONT_SIZE);
	private final Font MONOFONT = new java.awt.Font("Courier",
			Font.PLAIN, 12);

	/* SET GUI COLORS */
	private final java.awt.Color COLOR_1 = new java.awt.Color(255, 255, 255);
	private final java.awt.Color COLOR_2 = new java.awt.Color(0, 0, 0);
	private final java.awt.Color COLOR_3 = new java.awt.Color(212, 208, 200);

	// SWING COMPONENTS
	private final JTabbedPane mainTabbedPanel = new JTabbedPane();
	private final JPanel treePanel;
	private final JScrollPane jsp;
	private final JScrollPane stats;
	private final JTextArea statsTextArea = new JTextArea();

	private final JLabel headingLabel = new JLabel("Selected Node");
	private final JLabel keyLabel = new JLabel("Key:");
	private final JTextField keyDisplay = new JTextField();

	public TreeViewGUI( DefaultMutableTreeNode[] trees, String rootTag, String statsString ) {

		if(rootTag==null)this.rootTag="Simulation";
		else this.rootTag=rootTag;

		DefaultMutableTreeNode t = new DefaultMutableTreeNode(this.rootTag);
		for (DefaultMutableTreeNode tree : trees)
			t.add(tree);
	    final JTree jtree = new JTree(t);
	    jtree.setFont(MONOFONT);

		java.net.URL imgURL = TreeViewGUI.class.getResource("/uk/ac/standrews/cs/asa/resources/Pictures/tree.jpg");
		this.setSize(new Dimension(900, 700));
		setResizable(true);
		setBackground(COLOR_2);
		setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
		setTitle("CHORD TREE VIEWER");
		setFont(FONT);
		setForeground(COLOR_1);
		setIconImage(new ImageIcon(imgURL).getImage());

		// TABBED PANEL AND COMPONENTS
		mainTabbedPanel.setSize(900, 700);
		mainTabbedPanel.setBackground(COLOR_3);
		mainTabbedPanel.setForeground(COLOR_2);
		mainTabbedPanel.setFont(FONT);

		//jpanel to display tree etc.
		treePanel = new JPanel();
		treePanel.setLayout(new java.awt.GridBagLayout());
		//treePanel.setPreferredSize(new java.awt.Dimension(900, 700));
		//treePanel.setMinimumSize(new java.awt.Dimension(900, 700));
		treePanel.setFont(FONT);
		treePanel.setBackground(COLOR_1);

		// scrollpane to hold jtree
		jsp = new JScrollPane(jtree);
		jsp.setPreferredSize(new java.awt.Dimension(900, 500));
		jsp.setMinimumSize(new java.awt.Dimension(900, 500));
		jsp.setFont(MONOFONT);
		jsp.setBackground(COLOR_1);

		statsTextArea.setText("Simulation Statistics\n---------------------\n\n");
		statsTextArea.append(statsString);
		statsTextArea.setEditable(false);
		statsTextArea.setSelectionColor(COLOR_3);
		statsTextArea.setFont(MONOFONT);
		stats= new JScrollPane(statsTextArea);
		stats.setBackground(COLOR_1);
		stats.setPreferredSize(new java.awt.Dimension(700, 600));
		stats.setMinimumSize(new java.awt.Dimension(700, 600));



		//components
		headingLabel.setFont(FONT);
		headingLabel.setForeground(COLOR_2);

		keyLabel.setFont(FONT);
		keyLabel.setForeground(COLOR_2);

		keyDisplay.setPreferredSize(new java.awt.Dimension(400, 22));
		keyDisplay.setMinimumSize(new java.awt.Dimension(400, 22));
		keyDisplay.setMaximumSize(new java.awt.Dimension(400, 22));
		//errorMsg.setMargin(new java.awt.Insets(5, 0, 0, 0));
		keyDisplay.setFont(MONOFONT);
		keyDisplay.setEditable(false);

		//add components to treePanel
		treePanel.add(jsp, new java.awt.GridBagConstraints(0, 0, 2,
				1, 0.0, 0.0, java.awt.GridBagConstraints.NORTHWEST ,
				java.awt.GridBagConstraints.NONE,
				new java.awt.Insets(5, 5, 0, 5), 0, 0));
		treePanel.add(headingLabel, new java.awt.GridBagConstraints(0, 1, 2,
				1, 0.0, 0.0, java.awt.GridBagConstraints.NORTHWEST ,
				java.awt.GridBagConstraints.NONE,
				new java.awt.Insets(5, 5, 0, 5), 0, 0));
		treePanel.add(keyLabel, new java.awt.GridBagConstraints(0, 2, 1,
				1, 0.0, 0.0, java.awt.GridBagConstraints.NORTHWEST ,
				java.awt.GridBagConstraints.NONE,
				new java.awt.Insets(5, 5, 5, 5), 0, 0));
		treePanel.add(keyDisplay, new java.awt.GridBagConstraints(1, 2, 1,
				1, 1.0, 1.0, java.awt.GridBagConstraints.NORTHWEST ,
				java.awt.GridBagConstraints.NONE,
				new java.awt.Insets(5, 5, 5, 5), 0, 0));

		// ADD TABS TO TABBED PANEL
		if (imgURL != null) {
			mainTabbedPanel.addTab("Spanning tree", new javax.swing.ImageIcon(
					imgURL), treePanel);
			mainTabbedPanel.addTab("Statistics", new javax.swing.ImageIcon(
					imgURL), stats);
		} else {
			mainTabbedPanel.addTab("Spanning tree", jsp);
			mainTabbedPanel.addTab("Statistics", stats);
		}

		//ADD MAIN CONTAINERS TO THE FRAME
		getContentPane().add(mainTabbedPanel, java.awt.BorderLayout.CENTER);

		//SETUP THE MOUSE LISTENER
		MouseListener ml = new MouseAdapter() {
			@Override
			public void mousePressed(MouseEvent e) {
				int selRow = jtree.getRowForLocation(e.getX(), e.getY());
				TreePath selPath = jtree.getPathForLocation(e.getX(), e.getY());
				if (selRow != -1)
					mySingleClick(selRow, selPath);
			}

			private void mySingleClick(int selRow, TreePath selPath) {
				if(selPath.getPathCount()>1){
					selPath.getLastPathComponent();
					DefaultMutableTreeNode node = (DefaultMutableTreeNode)selPath.getLastPathComponent();
					Object[] path = node.getUserObjectPath();
					try{
						IP2PNode jcn = (IP2PNode)path[path.length-1];
						String key = jcn.getKey().keyValue().toString(16);
						if(key.length()<40)
							for(int i=0;i<40-key.length();i++)
								key="0"+key;
						keyDisplay.setText(key);
					}catch(ClassCastException e){
						e.printStackTrace();
						ErrorHandling.hardError("Node in tree was not a ChordNode as expected");
					} catch (Exception e) {
			            ErrorHandling.exceptionError(e, "ChordNode.getKey threw exception");
                    }
				}
			}
		};
		jtree.addMouseListener(ml);

		// DISPLAY GUI
		setVisible(true);
	}
}