/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 04-Dec-2004 at 16:34:25.
 */
package uk.ac.standrews.cs.nds.p2p.util;

import java.util.Comparator;

import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.interfaces.IP2PNode;
import uk.ac.standrews.cs.nds.util.ErrorHandling;


/**
 * Comparator class for comparing P2PNode objects via their keys.
 * 
 * @author al, stuart
 */
public class NodeComparator implements Comparator<IP2PNode> {

    public int compare(IP2PNode arg0, IP2PNode arg1) {
        if( arg1 == null ) {
            return 1;
        }
        if( arg0 == null ) {
            return -1;
        }
        IKey k0 = null;
        IKey k1 = null;
        try {
            k0 = arg0.getKey();
            k1 = arg1.getKey();
        } catch (Exception e1) {
            ErrorHandling.exceptionError( e1, "Exception getting key for NodeComparison" );
            // don't need to do anything here - next lines takes care of it.
        }
        if( k1 == null ) {
            return 1;
        }
        if( k0 == null ) {
            return -1;
        }
        return (k0.compareTo(k1));
    }
}
