/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.p2p.util;

import uk.ac.standrews.cs.nds.p2p.interfaces.IHash;

public class RandomGenerator {
	
	private boolean deterministic = false;
	
	private static int count = 0;
	
	private IHash hash;
	
	public RandomGenerator(IHash hash) {
		this.hash = hash;
	}
	
	public byte[] generateRandomBytes(int number_of_bytes) {
		
		int BYTES_IN_HASH = hash.getHashSize();
		
		byte[] bytes = new byte[number_of_bytes];
		
		int i = 0;
		for (; i < number_of_bytes / BYTES_IN_HASH; i++) {
			
			setRandomBytes(bytes, i, BYTES_IN_HASH);
		}
		
		// Fill in remaining bytes if necessary.
		int bytes_left = number_of_bytes % BYTES_IN_HASH;
		if (bytes_left > 0) {
			setRandomBytes(bytes, i, bytes_left);
		}
	
		return bytes;
	}

	private void setRandomBytes(byte[] bytes, int i, int bytes_to_copy) {
		
		String seed;
		
		if (deterministic) seed = "seed" + count++;
		else               seed = randomString();
		
		byte[] hashed = hash.hash(seed.getBytes());
		System.arraycopy(hashed, 0, bytes, i * hashed.length, bytes_to_copy);
	}
	
	public static String randomString() {
		
		return String.valueOf(System.currentTimeMillis()) + String.valueOf(Runtime.getRuntime().freeMemory());
	}

	public boolean isDeterministic() {
		return deterministic;
	}

	public void setDeterministic(boolean deterministic) {
		this.deterministic = deterministic;
	}
}
