/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 26-Oct-2004
 */
package uk.ac.standrews.cs.nds.p2p.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import uk.ac.standrews.cs.nds.p2p.interfaces.IHash;
import uk.ac.standrews.cs.nds.util.ErrorHandling;


/**
 * Wrapper for SHA1 hash implementation.
 *
 * @author stuart, graham
 */
public class SHA1Hash implements IHash {

	private MessageDigest sha1;

	public SHA1Hash() {

		try {
			sha1 = MessageDigest.getInstance("SHA1");
		} catch (NoSuchAlgorithmException e) {
			ErrorHandling.exceptionError(e, "couldn't load SHA1");
		}
	}

	/**
	 * Generates the hash of the given data.
	 *
	 * @param data the data to be hashed
	 * @return the result of hashing the data
	 */
	public synchronized byte[] hash(byte[] data) {

	 	sha1.reset();
		sha1.update(data, 0, data.length);
		return sha1.digest();
	}

	/**
	 * Returns the fixed length of the hash digest in bytes.
	 *
	 * @return the fixed length of the hash digest in bytes
	 */
	public int getHashSize() {

		return sha1.getDigestLength();
	}
}
