package uk.ac.standrews.cs.nds.util;

import java.util.ArrayList;
import java.util.Collection;

/**
 * Transforms collections of strings to arrays and vice versa.
 * 
 * @author Markus
 */
public class ArrayUtils {

	private static String DEFAULT_DELIMITER = ",";

	/**
	 * Transforms any collection of strings to a string, delimited with the
	 * default delimiter (,).
	 * 
	 * @param collection
	 *            the collection to transform
	 * @return the string
	 */
	public static String implode(Collection<String> collection) {
		return implode(collection, DEFAULT_DELIMITER);
	}

	/**
	 * Transforms any collection of strings to a string, delimited with a custom
	 * delimiter.
	 * 
	 * @param collection
	 *            the collection to transform
	 * @param delimiter
	 *            the custom delimiter used between the strings
	 * @return the string
	 */
	public static String implode(Collection<String> collection, String delimiter) {
		String string_representation_of_array = "";

		int i = 0;
		for (String string : collection) {
			if (i == 0) {
				string_representation_of_array = string;
			} else {
				string_representation_of_array = string_representation_of_array
						+ delimiter + string;
			}
			i++;
		}

		return string_representation_of_array;
	}

	/**
	 * Transforms a string to a collection if it contains delimiters (specified
	 * by the default delimiter). 
	 * 
	 * @param string
	 *            the string to split up
	 * @return the collection of strings
	 */
	public static Collection<String> explode(String string) {
		return explode(string, DEFAULT_DELIMITER);
	}

	/**
	 * Transforms a string to a collection if it contains delimiters. E.g.
	 * "amber,blue,yellow" -> Collection<String> = {amber, blue, yellow}
	 * 
	 * @param string
	 *            the string to split up
	 * @param delimiter
	 *            a custom delimiter
	 * @return the collection of strings
	 */
	public static Collection<String> explode(String string, String delimiter) {
		Collection<String> collection = new ArrayList<String>();

		String[] st = string.split(delimiter);
		for (String string2 : st) {
			collection.add(string2);
		}
		return collection;

	}
}
