/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Created on Aug 10, 2005 at 8:28:58 PM.
 */
package uk.ac.standrews.cs.nds.util;

import java.util.HashMap;
import java.util.Map;

/**
 * Utility for manipulating command line arguments.
 *
 * @author graham
 */
public class CommandLineArgs {

	/**
	 * Parses the given command line arguments into a map from flags to values.
	 * Each argument is assumed to be of the form
	 *
	 *   -xvalue
	 *
	 * which would result in a map entry from "-x" to "value"
	 *
	 * @param args the command line arguments
	 * @return a map from flags to values
	 */
	public static Map<String, String> parseCommandLineArgs(String[] args) {

		Map<String, String> map = new HashMap<String, String>();

		for (String element : args)
			if (element.length() > 1) {

				String flag = element.substring(0, 2);
				String value = element.substring(2);
				map.put(flag, value);
			}

		return map;
	}

	/**
	 * Extracts the argument value for the given flag from the given command line arguments.
	 *
	 * @param args the command line arguments
	 * @param flag the required flag
	 * @return the value for the given flag, or null if not found
	 */
	public static String getArg(String[] args, String flag) {

		return parseCommandLineArgs(args).get(flag);
	}

	/**
	 * Extracts an integer value for the given flag from the given command line arguments.
	 *
	 * @param args the command line arguments
	 * @param flag the required flag
	 * @param default_value the value to be returned if the flag is not found
	 * @return the resulting value
	 */
	public static int extractIntFromCommandLineArgs(String[] args, String flag, int default_value) {

		String value_string = getArg(args, flag);

		try {
			if (value_string != null) return Integer.parseInt(value_string);
			else                      return default_value;
		}
		catch (NumberFormatException e) {
			return default_value;
		}
	}
}
