/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.util;

import java.io.File;
import java.io.IOException;

public class FileUtil {

	/**
	 * Deletes a given file, and recursively deletes contents if it is a directory.
	 * 
	 * @param file a file or directory to be deleted
	 * @throws IOException if the file cannot be deleted
	 */
	public static void delete(File file) throws IOException {
		
		if (file.isDirectory()) {
			
			String[] children = file.list();
			if (children == null) throw new IOException("null directory listing");
			for (String child : children) delete(new File(file, child));
		}

		if (!file.delete()) throw new IOException("couldn't delete file " + file.getAbsolutePath());
	}
	
	public static File createFile(String s) {
//		Diagnostic.trace(DiagnosticLevel.RUN, "creating file: ", s);
		return new File(s);
	}
	
	public static File createFile(File f, String s) {
//		Diagnostic.trace(DiagnosticLevel.RUN, "creating file: ", f.getAbsolutePath(), " ", s);
		return new File(f, s);
	}
}
