/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 21-Jan-2005
 */
package uk.ac.standrews.cs.nds.util;


import java.io.PrintStream;
import java.text.DecimalFormat;

import uk.ac.standrews.cs.nds.p2p.simulation.interfaces.ISimulationFactory;

/**
 * Output class with suppport for singleton and redirection of I/O stream elsewhere
 * Abstracts over output streams
 * TODO - Graham this is a job for you - we need a witch hunt for System.out.println :)
 *
 * @author stuart
 */
public class Output {

	public static String DECIMAL_FORMAT_STRING = "#.###";
	public static String DEFAULT_SEPARATOR = "\t";
	private final DecimalFormat df;
	private final String separator;
	private PrintStream printStream;
	private static Output instance = null;

	private Output(){
		df=new DecimalFormat(DECIMAL_FORMAT_STRING);
		separator=DEFAULT_SEPARATOR;
		printStream = System.out;
	}

	public static Output getInstance() {
	    if( instance == null )
			instance = new Output();
	    return instance;
	}

	public void outputExperimentalSetup(int numNodes, ISimulationFactory<?> sf){
		printStream.println("Experimental setup:");
		printlnSeparated("Number of nodes",numNodes);
		printlnSeparated("ISimulationFactory implmentation",sf.getClass().getName());
	}

	public void println( String s ) {
		printStream.println(s);
	}

	public void print( String s ) {
		printStream.println(s);
	}

	public void println(double d) {
		printStream.println(d);
	}

	public void printlnSeparated(String s1, String s2){
		printStream.print(s1);
		printStream.print(separator);
		printStream.print(s2);
		printStream.println();
	}

	public void printlnSeparated(String s1, double d1){
		printStream.print(s1);
		printStream.print(separator);
		printStream.print(df.format(d1));
		printStream.println();
	}

	public void printlnSeparated(int s, double d) {
		printStream.print(s);
		printStream.print(separator);
		printStream.print(df.format(d));
		printStream.println();
	}


	public void printlnSeparated(String s1, int i1){
		printStream.print(s1);
		printStream.print(separator);
		printStream.print(i1);
		printStream.println();
	}


	public void printlnSeparated(String s1, String s2, String s3, String s4, String s5, String s6) {
		printStream.print(s1);
		printStream.print(separator);
		printStream.print(s2);
		printStream.print(separator);
		printStream.print(s3);
		printStream.print(separator);
		printStream.print(s4);
		printStream.print(separator);
		printStream.print(s5);
		printStream.print(separator);
		printStream.print(s6);
		printStream.println();
	}

	public void printlnSeparated(String s1, String s2, String s3, String s4, String s5, String s6, String s7) {
		printStream.print(s1);
		printStream.print(separator);
		printStream.print(s2);
		printStream.print(separator);
		printStream.print(s3);
		printStream.print(separator);
		printStream.print(s4);
		printStream.print(separator);
		printStream.print(s5);
		printStream.print(separator);
		printStream.print(s6);
		printStream.print(separator);
		printStream.print(s7);
		printStream.println();
	}

	public void printlnSeparated(int i1, double d1, double d2, double d3, double d4, double d5) {
		printStream.print(i1);
		printStream.print(separator);
		printStream.print(df.format(d1));
		printStream.print(separator);
		printStream.print(df.format(d2));
		printStream.print(separator);
		printStream.print(df.format(d3));
		printStream.print(separator);
		printStream.print(df.format(d4));
		printStream.print(separator);
		printStream.print(df.format(d5));
		printStream.println();
	}

	public void printlnSeparated(int i1, double d1, double d2, double d3, double d4, double d5 , double d6) {
		printStream.print(i1);
		printStream.print(separator);
		printStream.print(df.format(d1));
		printStream.print(separator);
		printStream.print(df.format(d2));
		printStream.print(separator);
		printStream.print(df.format(d3));
		printStream.print(separator);
		printStream.print(df.format(d4));
		printStream.print(separator);
		printStream.print(df.format(d5));
		printStream.print(separator);
		printStream.print(df.format(d6));
		printStream.println();
	}

    /**
     * @return Returns the out.
     */
    public PrintStream getPrintStream() {
        return printStream;
    }
    /**
     * @param out The out to set.
     */
    public void setPrintStream(PrintStream out) {
        printStream = out;
    }
}
