/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.ac.standrews.cs.nds.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

public class Permutation implements Iterable<Integer> {

	private static final long SEED = 25234676276L;
	private static final Random random = new Random(SEED);

	private final List<Integer> list;

	public Permutation(int size) {

		list = new ArrayList<Integer>(size);

		for (int i = 0; i < size; i++) list.add(i);
		mixUp();
	}

	public void fix(int index, int element) {

		if (index > list.size()) throw new IndexOutOfBoundsException();
		int current_element_index = list.indexOf(element);
		swap(index, current_element_index);
	}

	public Iterator<Integer> iterator() {

		return list.iterator();
	}

	private void mixUp() {

		int size = list.size();

		for (int i = 0; i < size; i++) {
			int a = random.nextInt(size);
			int b = random.nextInt(size);
			swap(a, b);
		}
	}

	private void swap(int a, int b) {

		int temp = list.get(a);
		list.set(a, list.get(b));
		list.set(b, temp);
	}

	public static void main(String[] args) {

		Permutation p = new Permutation(10);
		for (int i : p) System.out.println(i);
		System.out.println();

		p.fix(0, 9);
		p.fix(1, 3);
		for (int i : p) System.out.println(i);
	}
}
