/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Created on Jun 2, 2006 at 2:44:06 PM.
 */
package uk.ac.standrews.cs.nds.util;

import java.util.HashMap;
import java.util.Map;


/**
 *
 *
 * @author graham
 */
public enum PlatformDescriptor {

	PLATFORM_MAC,

	PLATFORM_LINUX,

	PLATFORM_WINDOWS;

	public static PlatformDescriptor getPlatform() {

		String platform_name = System.getProperty("os.name");

		if (platform_name.equals(NAME_MAC))         return PLATFORM_MAC;
		if (platform_name.equals(NAME_LINUX))       return PLATFORM_LINUX;
		if (platform_name.startsWith(NAME_WINDOWS)) return PLATFORM_WINDOWS;

		ErrorHandling.hardError("unknown platform: ", platform_name);
		return null;
	}

	public static PlatformDescriptor getPlatform(SSH2ConnectionWrapper connection_wrapper) {

		if (cache.containsKey(connection_wrapper)) return cache.get(connection_wrapper);

		// "Darwin" / "Linux" / no reply

		final StringWrapper wrapper = PLATFORM_MAC.new StringWrapper();   // Horrible Java hack.
		//TODO stu - change this to use java.util.concurrent.Semaphore
        final Semaphore sync = new Semaphore(0);

		// Processor to read the output from the remote process and extract the platform name.
		IStreamProcessor extract_platform = new IStreamProcessor() {

			// First word in reply should be platform name.

			String uname_output = "";
			boolean done = false;

			public boolean processByte(int byte_value) {

				if (!done) {

					// Test for space character.
					if (byte_value == ' ') {
						wrapper.wrapped = uname_output;
						sync.semSignal();
						done = true;
					}

					// Accumulate the output.
					uname_output += (char)byte_value;
				}

				return false;
			}
		};

		// Clone the SSH connection to avoid interference with other use of this one.
		try {
	        Processes.runProcess(UNAME_COMMAND, connection_wrapper.clone(), System.out, System.err, extract_platform, null);
        }
		catch (Exception e) {
	        ErrorHandling.exceptionError(e, "cloning SSH connection");
        }

		sync.semWait();

		PlatformDescriptor result = PlatformDescriptor.PLATFORM_WINDOWS;

		if (wrapper.wrapped.equals(UNAME_REPLY_MAC))   result = PlatformDescriptor.PLATFORM_MAC;
		if (wrapper.wrapped.equals(UNAME_REPLY_LINUX)) result = PlatformDescriptor.PLATFORM_LINUX;

		cache.put(connection_wrapper, result);

		return result;
	}

	private static final String UNAME_REPLY_LINUX = "Linux";
	private static final String UNAME_REPLY_MAC = "Darwin";

	private static final String UNAME_COMMAND = "uname -a";

	private static final String NAME_MAC =	   "Mac OS X";
	private static final String NAME_LINUX =   "Linux";
	private static final String NAME_WINDOWS = "Windows";

	private static Map<SSH2ConnectionWrapper, PlatformDescriptor> cache = new HashMap<SSH2ConnectionWrapper, PlatformDescriptor>();

    private class StringWrapper {

    	String wrapped = null;
	}
}
