/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Created on Jun 6, 2006 at 1:39:58 PM.
 */
package uk.ac.standrews.cs.nds.util;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;

import com.mindbright.jca.security.SecureRandom;
import com.mindbright.ssh2.SSH2Exception;
import com.mindbright.ssh2.SSH2SimpleClient;
import com.mindbright.ssh2.SSH2Transport;
import com.mindbright.util.RandomSeed;
import com.mindbright.util.SecureRandomAndPad;

/**
 * Wrapper for SSH2 connection details.
 *
 * @author graham
 */
public class SSH2ConnectionWrapper {

	/**
	 * Default port for SSH connection.
	 */
	private static final int SSH_PORT = 22;

	private final InetAddress server;
	private final String user_name;

	private String password, key_file, key_password;

	private SSH2SimpleClient client = null;

	public SSH2ConnectionWrapper(InetAddress server, String user_name, String key_file, String key_password) {

		this.server = server;
		this.user_name = user_name;
		this.key_file = key_file;
		this.key_password = key_password;
	}

	public SSH2ConnectionWrapper(InetAddress server, String user_name, String password) {

		this.server = server;
		this.user_name = user_name;
		this.password = password;
 	}

	private SSH2ConnectionWrapper(InetAddress server, String user_name, String password, String key_file, String key_password) {

		this.server = server;
		this.user_name = user_name;
		this.password = password;
		this.key_file = key_file;
		this.key_password = key_password;
	}

	public SSH2SimpleClient getSSH2Client() throws SSH2Exception, IOException {

		if (client == null) client = makeSSH2Client();
		return client;
	}

	@Override
	public SSH2ConnectionWrapper clone() {

		return new SSH2ConnectionWrapper(server, user_name, password, key_file, key_password);
	}

	private SSH2SimpleClient makeSSH2Client() throws SSH2Exception, IOException {

		Socket server_socket = new Socket(server, SSH_PORT);
		byte[] seed = RandomSeed.getSystemStateHash();
		SecureRandomAndPad secure_random = new SecureRandomAndPad(new SecureRandom(seed));

		SSH2Transport transport = new SSH2Transport(server_socket, secure_random);

		if (password != null) return new SSH2SimpleClient(transport, user_name, password);
		else                  return new SSH2SimpleClient(transport, user_name, key_file, key_password);
	}
}
