/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on Apr 27, 2005 at 1:23:48 PM.
 */
package uk.ac.standrews.cs.nds.util.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import org.junit.Test;

import uk.ac.standrews.cs.nds.eventModel.eventBus.EventBus;
import uk.ac.standrews.cs.nds.eventModel.eventBus.busInterfaces.IEventBus;
import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;

/**
 * Test class for Diagnostic.
 *
 * @author graham
 */
public class DiagnosticTest {

	/**
	 *
	 */
	@Test
    public void diagnostic() {

        IEventBus bus = new EventBus();

        DiagnosticTestAdapter adapter = new DiagnosticTestAdapter();

        Diagnostic.setBus(bus);

        bus.register(adapter);

        Diagnostic.setLevel(DiagnosticLevel.RUN);
        Diagnostic.setLocalErrorReporting(false);

        // Shouldn't get a message with diagnostic level lower than global level.
        Diagnostic.trace(DiagnosticLevel.FULL);
        assertNull(adapter.event);

        // Should get a message with diagnostic level same as global level.
        Diagnostic.trace(DiagnosticLevel.RUN, "test1");
        assertNotNull(adapter.event);
        assertEquals(DiagnosticTest.class.getName() + "::diagnostic : test1", adapter.event.get("msg"));

        // Should get a message with diagnostic level higher than global level.
        Diagnostic.trace(DiagnosticLevel.NONE, "test2");
        assertEquals(DiagnosticTest.class.getName() + "::diagnostic : test2", adapter.event.get("msg"));
    }
}
