/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on Dec 15, 2004 at 10:56:29 PM.
 */
package uk.ac.standrews.cs.nds.util.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.net.InetSocketAddress;

import org.junit.Test;

import uk.ac.standrews.cs.nds.p2p.interfaces.IKey;
import uk.ac.standrews.cs.nds.p2p.util.HashBasedKeyFactory;
import uk.ac.standrews.cs.nds.p2p.util.SHA1KeyFactory;

/**
 * Test class for KeyFactory.
 *
 * @author graham
 */
public class KeyFactoryTest {

	private final HashBasedKeyFactory key_factory = new SHA1KeyFactory();

    /**
     *
     */
    @Test
    public void generateKeyWithString() {

        IKey k1 = key_factory.generateKey("null");
        IKey k2 = key_factory.generateKey("quick brown fox");
        IKey k3 = key_factory.generateKey("quick brown fox");

        // Key generated by SHA1 from string "null".
        assertEquals("2be88ca4242c76e8253ac62474851065032d6833", k1.toString());

        // Key generated by SHA1 from string "quick brown fox".
        assertEquals("a9762606f9e33e452f06b4562e253efb6038b512", k2.toString());

        // Subsequent calls should return equal keys.
        assertTrue(k2.compareTo(k3) == 0);
        assertFalse(k1.compareTo(k2) == 0);
    }

    /**
     *
     */
    @Test
    public void generateKeyWithIPAddress() {

        IKey k1 = key_factory.generateKey(new InetSocketAddress("138.251.195.25", 80));
        IKey k2 = key_factory.generateKey(new InetSocketAddress("212.58.224.56", 1024));

        // Keys generated by SHA1 from addresses above.
        assertEquals("195e24370036a4062b0d325c03153a2b9fc9c92e", k1.toString());
        assertEquals("6280b63d4e1f488235bce11c30c6d3130141fc07", k2.toString());
    }
}
