/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * Created on May 10, 2006 at 9:49:43 PM.
 */
package uk.ac.standrews.cs.nds.util.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.net.InetSocketAddress;
import java.net.UnknownHostException;

import org.junit.Test;

import uk.ac.standrews.cs.nds.util.NetworkUtil;

/**
 * @author graham
 */
public class NetworkTest {

	/**
	 * @throws NumberFormatException
	 */
	@Test
	public void testProcessHostPortParameter() throws NumberFormatException {

        InetSocketAddress addr;

        try {

	        addr = NetworkUtil.processHostPortParameter("edradour.cs.st-andrews.ac.uk:1010", 80);
	        assertEquals(addr.getHostName(), "edradour.cs.st-andrews.ac.uk");
	        assertEquals(addr.getPort(), 1010);

	        addr = NetworkUtil.processHostPortParameter("edradour.cs.st-andrews.ac.uk:1010", 80);
	        assertEquals(addr.getHostName(), "edradour.cs.st-andrews.ac.uk");
	        assertEquals(addr.getPort(), 1010);

	        addr = NetworkUtil.processHostPortParameter("edradour.cs.st-andrews.ac.uk:", 80);
	        assertEquals(addr.getHostName(), "edradour.cs.st-andrews.ac.uk");
	        assertEquals(addr.getPort(), 80);

	        addr = NetworkUtil.processHostPortParameter("edradour.cs.st-andrews.ac.uk", 80);
	        assertEquals(addr.getHostName(), "edradour.cs.st-andrews.ac.uk");
	        assertEquals(addr.getPort(), 80);

	        addr = NetworkUtil.processHostPortParameter(":1010", 80);
	        assertTrue(addr.getAddress().isSiteLocalAddress() || addr.getAddress().isLoopbackAddress());
	        assertEquals(addr.getPort(), 1010);
        }
        catch (UnknownHostException e) {
        	// Occurs when running tests disconnected from network.
        }
	}

	/**
	 *
	 */
	@Test
	public void testExtractHostName() {

		assertEquals(NetworkUtil.extractHostName("edradour.cs.st-andrews.ac.uk:1010"), "edradour.cs.st-andrews.ac.uk");
		assertEquals(NetworkUtil.extractHostName("edradour:1010"), "edradour");
		assertEquals(NetworkUtil.extractHostName("edradour:"), "edradour");
		assertEquals(NetworkUtil.extractHostName("edradour"), "edradour");
		assertEquals(NetworkUtil.extractHostName(":1010"), "");
	}

	/**
	 *
	 */
	@Test
	public void testExtractPortNumber() {

		assertEquals(NetworkUtil.extractPortNumber("edradour.cs.st-andrews.ac.uk:1010"), 1010);
		assertEquals(NetworkUtil.extractPortNumber("edradour:1010"), 1010);
		assertEquals(NetworkUtil.extractPortNumber("edradour:"), NetworkUtil.UNDEFINED_PORT);
		assertEquals(NetworkUtil.extractPortNumber("edradour"), NetworkUtil.UNDEFINED_PORT);
		assertEquals(NetworkUtil.extractPortNumber(":1010"), 1010);
	}
}
