/*
 *  NDS Library
 *  Copyright (C) 2006-2008 Distributed Systems Architecture Research Group
 *  http://www-systems.cs.st-andrews.ac.uk/
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * Created on 16-Feb-2005
 */
package uk.ac.standrews.cs.nds.util.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;

import uk.ac.standrews.cs.nds.util.Processes;
import uk.ac.standrews.cs.nds.util.SSH2ConnectionWrapper;

import com.mindbright.ssh2.SSH2Exception;

/**
 * @author graham
 */
public class ProcessesTest {

	/**
	 * @param args
	 * @throws InterruptedException
	 * @throws SSH2Exception
	 * @throws IOException
	 */
	public static void main(String[] args) throws InterruptedException, IOException, SSH2Exception {

		System.out.println("running local process:");
		testRunProcess();

		System.out.println("\nrunning remote process:");
		testRunRemoteProcess();

		System.out.println("\nrunning remote Java process:");
		testRunRemoteJavaProcess();
	}

	/**
	 * @throws InterruptedException
	 * @throws IOException
	 *
	 */
	public static void testRunProcess() throws InterruptedException, IOException {

		Process p = Processes.runProcess("uname -a");
		p.waitFor();
		p.destroy();
	}

	/**
	 * @throws IOException
	 * @throws SSH2Exception
	 * @throws InterruptedException
	 *
	 */
	public static void testRunRemoteProcess() throws IOException, SSH2Exception, InterruptedException {

		String command = "/usr/java/bin/java -version";
		InetAddress server = InetAddress.getByName("cordelia.cs.st-andrews.ac.uk");

		String user = "graham";
		String privateKeyFilePath = "/Users/graham/.ssh/id_rsa";

		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		System.out.print("enter ssh passphrase :");

		String passPhrase = reader.readLine();

		SSH2ConnectionWrapper client = new SSH2ConnectionWrapper(server, user, privateKeyFilePath, passPhrase);
		Process p = Processes.runProcess(command, client);
		p.waitFor();
		p.destroy();
	}

	/**
	 * @throws IOException
	 * @throws SSH2Exception
	 * @throws InterruptedException
	 *
	 */
	public static void testRunRemoteJavaProcess() throws IOException, SSH2Exception, InterruptedException {

		InetAddress server = InetAddress.getByName("cordelia.cs.st-andrews.ac.uk");

		String user = "graham";
		String privateKeyFilePath = "/Users/graham/.ssh/id_rsa";

		BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
		System.out.print("enter ssh passphrase :");

		String passPhrase = reader.readLine();

		SSH2ConnectionWrapper client = new SSH2ConnectionWrapper(server, user, privateKeyFilePath, passPhrase);
		Process p = Processes.runJavaProcess(RemoteTestClass.class, client);
		p.waitFor();
		p.destroy();
	}
}
